//
// NeoMagic.h
// NeoMagic Display Driver
//
// Copyright (c) 2004 David Lazaro Saz.
// All rights reserved.
//

#ifndef NEOMAGIC_H__
#define NEOMAGIC_H__

#import <driverkit/IOFrameBufferDisplay.h>

#import "VGA.h"


/*
 * NeoMagic Chipsets
 */
typedef enum {
    NM2070 = 0,
    NM2090,
    NM2093,
    NM2097,
    NM2160,
    NM2200,
    NM2230,
    NM2360,
    NM2380
} NeoMagicChipset;


/*
 * NeoMagic Extended Registers
 */
#define NEO_EXT_CRTC_MAX     0x85
#define NEO_EXT_GRAPHICS_MAX 0xC7

typedef struct {
    unsigned char useExtendedRegisters;
    unsigned char CRTC[NEO_EXT_CRTC_MAX + 1];
    unsigned char graphics[NEO_EXT_GRAPHICS_MAX + 1];
    
    unsigned char generalLock;
    unsigned char externalCRTDisplayAddress;
    unsigned char externalCRTOffset;
    unsigned char systemInterfaceControl1;
    unsigned char systemInterfaceControl2;
    unsigned char extendedColorModeSelect;
    unsigned char singleAddressPage;
    unsigned char dualAddressPage;
    unsigned char biosMode;
    unsigned char panelDisplayControl1;
    unsigned char panelDisplayControl2;
    unsigned char panelDisplayControl3;
    unsigned char panelVerticalCenter1;
    unsigned char panelVerticalCenter2;
    unsigned char panelVerticalCenter3;
    unsigned char panelVerticalCenter4;
    unsigned char panelVerticalCenter5;
    unsigned char panelHorizontalCenter1;
    unsigned char panelHorizontalCenter2;
    unsigned char panelHorizontalCenter3;
    unsigned char panelHorizontalCenter4;
    unsigned char panelHorizontalCenter5;
    unsigned char sequencer1;
    unsigned char programVCLK;
    unsigned char VCLK3NumeratorLow;
    unsigned char VCLK3NumeratorHigh;
    unsigned char VCLK3Denominator;
    unsigned char verticalExtension;
} NeoMagicRegisters;


@interface NeoMagic : IOFrameBufferDisplay
{
    // The information for the selected display mode.
    const IODisplayInfo *displayMode;
    const VGAMode       *vgaMode;

    // Adapter information
    NeoMagicChipset chipset;
    NeoMagicRegisters *registers;
    
    int panelWidth;
    int panelHeight;
    
    // VGA
    VGADescription *VGA;
    VGARegisters   *vgaRegisters;
    
    // Saved Registers
    VGARegisters      *savedVGARegisters;
    NeoMagicRegisters *savedRegisters;
    
    // Transfer Tables
    unsigned char *redTransferTable;
    unsigned char *greenTransferTable;
    unsigned char *blueTransferTable;
    
    int transferTableCount;

    // Screen Brightness
    int brightnessLevel;
}

@end

#endif  /* NEOMAGIC_H__ */
